@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # coding: utf-8
 
 """
@@ -9,7 +8,7 @@ from __future__ import absolute_import
 
 import logging
 
-__version__ = "2.2.1"
+__version__ = "3.1.1"
 
 log = logging.getLogger('can')
 
@@ -22,7 +21,12 @@ class CanError(IOError):
     """
     pass
 
+
 from .listener import Listener, BufferedReader, RedirectReader
+try:
+    from .listener import AsyncBufferedReader
+except ImportError:
+    pass
 
 from .io import Logger, Printer, LogReader, MessageSync
 from .io import ASCWriter, ASCReader
@@ -34,14 +38,14 @@ from .io import SqliteWriter, SqliteReader
 from .util import set_logging_level
 
 from .message import Message
-from .bus import BusABC
+from .bus import BusABC, BusState
 from .thread_safe_bus import ThreadSafeBus
 from .notifier import Notifier
 from .interfaces import VALID_INTERFACES
 from . import interface
 from .interface import Bus, detect_available_configs
 
-from can.broadcastmanager import send_periodic, \
+from .broadcastmanager import send_periodic, \
     CyclicSendTaskABC, \
     LimitedDurationCyclicSendTaskABC, \
     ModifiableCyclicTaskABC, \
